#version 330 core

in vec3 fc;
in vec2 fuv;
in vec3 position_worldSpace;
in vec3 viewDirection_cameraSpace;
in vec3 lightDirection_cameraSpace;
in vec3 normal_cameraSpace;
in vec3 normalInterpolated_cameraSpace;
in vec4 shadow_Position;

out vec3 outputF;

uniform vec3 lightPosition_worldSpace;
uniform sampler2D textureSampler;
uniform sampler1D texture1DSampler;
uniform sampler2DShadow shadowMap;

vec2 poissonDisk[4] = vec2[](
  vec2( -0.94201624, -0.39906216 ),
  vec2( 0.94558609, -0.76890725 ),
  vec2( -0.094184101, -0.92938870 ),
  vec2( 0.34495938, 0.29387760 )
);

void main()
{
		if(dot(viewDirection_cameraSpace, normal_cameraSpace) < 0)
	{
		outputF= vec3(0,0,0);
	} else {
		//TODO: Hardcoded!
		//Light intensity and color
		vec3 lightColor = vec3(1.0,1.0,1.0);
		float lightPower = 100.0f;

		// Material properties
		vec3 materialDiffuseColor = texture2D( textureSampler, fuv ).rgb;
		vec3 materialAmbientColor = vec3(0.1,0.1,0.1) * materialDiffuseColor;
		vec3 materialSpecularColor = vec3(0.3,0.3,0.3);

		//Distance to light
		float distanceLight = length(lightPosition_worldSpace - position_worldSpace);

		//calculate angle
		vec3 normalN = normalize(normalInterpolated_cameraSpace);
		vec3 lightN = normalize(lightDirection_cameraSpace);

		float cosTheta = clamp(dot(normalN,lightN) + 0.05, 0, 1);

		vec3 viewDirectionN = normalize(viewDirection_cameraSpace);
		vec3 reflection = reflect(-lightN,normalN);

		float cosAlpha = clamp(dot(viewDirectionN, reflection), 0, 1);

		vec3 ambient = materialAmbientColor;
		vec3 diffuse = materialDiffuseColor * lightColor * lightPower * cosTheta / pow(distanceLight, 2);
		vec3 specular = materialSpecularColor * lightColor * lightPower * pow(cosAlpha, 5) / pow(distanceLight, 2);

		outputF = ambient + diffuse + specular;
		

		float intensity = clamp(0.1 + cosTheta + cosAlpha, 0, 0.99);
		//read from shadow map if in shadow

		//bias against shadow acne
		float bias = 0.006*tan(acos(cosTheta)); // cosTheta is dot( n,l ), clamped between 0 and 1
		bias = clamp(bias, 0.006,0.01);

		float visibility = 1.0;
		for (int i=0;i<4;i++){
			int index = i;
			visibility -= 0.2*(1.0-texture( shadowMap, vec3(shadow_Position.xy + poissonDisk[index]/700.0,  (shadow_Position.z-bias)/shadow_Position.w) ));
		}

		outputF = texture(texture1DSampler, intensity ).rgb * materialDiffuseColor * visibility;

		//outputF = ambient + visibility*diffuse + visibility*specular;

		//outputF = visibility * materialDiffuseColor * lightColor;
		}
}