#version 330 core

in vec3 position;
in vec2 uv;
in vec3 color;
in vec3 normal;
in vec3 interpolatedNormal;

out vec3 fc;
out vec2 fuv;
out vec3 position_worldSpace;
out vec3 viewDirection_cameraSpace;
out vec3 lightDirection_cameraSpace;
out vec3 normal_cameraSpace;
out vec3 normalInterpolated_cameraSpace;
out vec4 shadow_Position;

uniform mat4 MVP;
uniform mat4 depthBiasMVP;
uniform  mat4 V;
uniform mat4 M;
uniform vec3 lightPosition_worldSpace;

void main()
{
	//compute transformed vertex coordinates
	gl_Position = MVP * vec4(position, 1.0);
	//compute shadow map coordinates
	shadow_Position = depthBiasMVP * vec4(position, 1.0);
	//compute world coordinates
	position_worldSpace = (M * vec4(position, 1.0)).xyz;
	//view direction of camera
	vec3 vertexPosition_cameraSpace = (V * M * vec4(position, 1.0)).xyz;
	viewDirection_cameraSpace = vec3(0,0,0) - vertexPosition_cameraSpace;
	//light direction
	vec3 lightPosition_cameraSpace = (V * vec4(lightPosition_worldSpace,1)).xyz;
	lightDirection_cameraSpace = lightPosition_cameraSpace + viewDirection_cameraSpace;
	//normals in cameraSpace
	normal_cameraSpace = (V * M * vec4(normal, 0)).xyz;
	normalInterpolated_cameraSpace = (V * M * vec4(interpolatedNormal, 0)).xyz;
	//pass color	
	fc = color;
	//pass texture coordinates
	fuv = uv;

	//Make objects front sides a bit smaller for black outline
	if(dot(viewDirection_cameraSpace, normal_cameraSpace) > 0)
	{
		gl_Position = MVP * vec4(position-0.04*interpolatedNormal, 1.0);
	}
}